VERSION = "0.01"
AUTHOR = "martblek (martblek@gmail.com)"

vb = renoise.ViewBuilder()
vbs = vb.views
dialog = nil

renoise.tool():add_menu_entry{
    name = "Main Menu:Tools: LowResource",
    invoke = function()
        prepare_for_start()
    end
}

renoise.tool():add_keybinding{
    name="Global:Tools: LowResource",
    invoke=function()
        prepare_for_start()
    end
}

_AUTO_RELOAD_DEBUG = function()
end

Notifiers = {}
function Notifiers:add(observable, callable)
    if not observable:has_notifier(callable) then
        observable:add_notifier(callable)
    end
end

function Notifiers:remove(observable, callable)
    if observable:has_notifier(callable) then
        observable:remove_notifier(callable)
    end
end

function calc_samples_size()
    local size = 0
    for i=1, #renoise.song().instruments do
        local instrument = renoise.song():instrument(i)
        for j = 1, #instrument.samples do
            local sample = instrument:sample(j)
            local buffer = sample.sample_buffer
            if buffer.has_sample_data then
                local samples = buffer.number_of_frames
                local samplerate = buffer.sample_rate
                local bitdepth = buffer.bit_depth
                local channels = buffer.number_of_channels
                -- calc duration
                local duration = samples / samplerate  -- in s
                -- why my calculations are off by 68 bytes when Renoise save sample ???
                local bytes = (duration * samplerate * channels * bitdepth) / 8 + 44 + 68
                size = size + bytes
            end
        end
    end
    local perc = (size / vbs.limit.value) * 100
    renoise.app():show_status('Total SampleLen: '..size..' bytes. Now on '..perc..' %')
end

function tool_idle()
  	if not dialog.visible then
    	-- disable notifiers
  	end
    
    if vbs.notifiers.value == true then
        calc_samples_size()
    end

end

function key_handler(dialog, key)
	  return key
end

function prepare_for_start()
  	if(dialog and dialog.visible) then
    	  dialog:show()
    	  return
  	end
  	dialog = renoise.app():show_custom_dialog('~ LowResource v'..tostring(VERSION)..' ~', make_gui, key_handler)
end

make_gui = vb:column{
    style='group',
    margin=4,
    spacing = 4,
    vb:text{text='Set limit in bytes'},
    vb:horizontal_aligner{
        mode='left',
        vb:slider{
            id='limit',
            width=120,
            min=100,
            max=2000000,
            value=128000,
            notifier=function(value)
                vbs.field.value=value
            end
        },
        vb:valuefield{
            id='field',
            width=70,
            min=100,
            max=2000000,
            value=128000,
            notifier=function(value)
                vbs.limit.value=value
            end
        },
    },
    vb:horizontal_aligner{
        mode='left',
        vb:checkbox{
            id='notifiers',
            value=false,
            notifier=function(bool)
                if bool then
                    Notifiers:add(renoise.tool().app_idle_observable, tool_idle)
                    renoise.app():show_status('SampleLen limits ON.')
                else
                    Notifiers:remove(renoise.tool().app_idle_observable, tool_idle)
                    renoise.app():show_status('SampleLen limits OFF.')
                end
            end
        },
    },

}